#include <Adafruit_GFX.h>
#include <Adafruit_SSD1331.h>
#include <SPI.h>
#include <Wire.h>
#include <UTF8toSJIS.h>
#include <ShinonomeFONTread.h>

// The SSD1331 is connected like this (plus VCC plus GND) GPIO Pin
const uint8_t   OLED_pin_scl_sck        = 14; //13;
const uint8_t   OLED_pin_sda_mosi       = 13; //11;
const uint8_t   OLED_pin_cs_ss          = 15; //10;
const uint8_t   OLED_pin_res_rst        = 2; //9;
const uint8_t   OLED_pin_dc_rs          = 0; //8;

// SSD1331 color definitions
const uint16_t  TFT_BLACK        = 0x0000; //0
const uint16_t  TFT_BLUE         = 0x001F; //1
const uint16_t  TFT_RED          = 0xF800; //2
const uint16_t  TFT_GREEN        = 0x07E0; //3
const uint16_t  TFT_CYAN         = 0x07FF; //4
const uint16_t  TFT_MAGENTA      = 0xF81F; //5
const uint16_t  TFT_YELLOW       = 0xFFE0; //6
const uint16_t  TFT_WHITE        = 0xFFFF; //7

const uint8_t panelW=64;
const uint8_t panelH=16;
const uint8_t dTime=0;

const char* UTF8SJIS_file = "/Utf8Sjis.tbl"; //UTF8 Shift_JIS 変換テーブルファイル名を記載しておく
const char* ZenkakuFontFile = "/shnmk16.bdf"; //全角フォントファイル名を定義
const char* HalfFontFile = "/shnm8x16r.bdf"; //半角フォントファイル名を定義

String msg[9]= {"上演中は","お静かに","お願いし","ます。  ","閑かさや","  岩にし","み入る  ","蝉の声  ","（芭蕉）"};
uint8_t col[36]={7,7,7,7,  7,7,7,7,  7,7,7,7,  7,7,7,7,  1,1,1,1,  3,3,3,3, 3,3,3,3, 2,6,6,6, 5,5,5,5};
uint8_t msgN=9;
uint8_t msgL=4;
uint8_t sj_txt[32]; //Shift_JISコード
uint16_t sj_length; //Shift_JISコードの長さ
uint8_t font_buf[32][16]; //東雲フォントデータ
uint8_t tmp_buf1[16],tmp_buf2[16]; //フォントデータ９０度回転後の一時データ
uint8_t buf[16]={0}; 
uint8_t cp; //ある一文字のShift_JISコードバイト数
uint16_t x,y,i,cl,xb;
uint8_t dot[128][16]={0};
uint8_t xo=15;
uint8_t yo=22;
uint8_t ptr=0;
uint8_t msgPtr=0;
uint16_t tcolor[8]={TFT_BLACK,TFT_BLUE,TFT_RED,TFT_GREEN,TFT_CYAN,TFT_MAGENTA,TFT_YELLOW,TFT_WHITE};

uint8_t cnt=0;


// The colors we actually want to use
uint16_t        OLED_Text_Color         = TFT_WHITE;
uint16_t        OLED_Backround_Color    = TFT_BLUE;

// declare the display
Adafruit_SSD1331 display =
    Adafruit_SSD1331(
        OLED_pin_cs_ss,
        OLED_pin_dc_rs,
        OLED_pin_sda_mosi,
        OLED_pin_scl_sck,
        OLED_pin_res_rst
     );
UTF8toSJIS u8ts;
ShinonomeFONTread SFR;

void setup() {
  // initialise the SSD1331
  display.begin();
  delay(500);
  display.fillScreen(TFT_BLACK);
  display.setTextColor(OLED_Text_Color);
  display.setTextSize(1);
  display.setCursor(20,5); display.print("LED BOARD");
  display.setCursor(20,47); display.print("ADCL  (C)");
  display.drawRect(xo-2,yo-2,panelW+4,panelH+4,TFT_GREEN);
  getMessage(0);
  getMessage(64);
}

void getMessage(uint8_t p){
  u8ts.UTF8_to_SJIS_str_cnv(UTF8SJIS_file, msg[msgPtr], sj_txt, &sj_length);
  i=0;
  if (cnt==msgN*msgL) cnt=0;
  while(i<sj_length){
    cp=SFR.SjisToShinonome16FontRead(ZenkakuFontFile, HalfFontFile, 0, 0, sj_txt[i], sj_txt[i+1], font_buf[i], font_buf[i+1]);
    i=i+cp;
  }
  for (i=0; i<msgL;i++){
    for(y=0; y<panelH; y++){
      for (x=0; x<8; x++){
        dot[x+p+i*16][y]=bitRead(font_buf[2*i][y],7-x)*10+col[cnt];            
      }
      for (x=8; x<16; x++){
        dot[x+p+i*16][y]=bitRead(font_buf[2*i+1][y],15-x)*10+col[cnt];
      }      
    }
    cnt++;
  }
  msgPtr++;;
  if(msgPtr==msgN) {
    msgPtr=0;
  }
}

void dispMessage(uint8_t p){
  i=0;
  for (x=p; x<p+64; x++){
    xb=x; if (x>127) xb=x-128;
    for (y=0; y<panelH; y++){
      cl=dot[xb][y] % 10;
      if (dot[xb][y]>=10) display.drawPixel(xo+i,yo+y,tcolor[cl]);
      else display.drawPixel(xo+i,yo+y,TFT_BLACK);
    }
    i++; 
  }
  delay(dTime);
}
void loop() {
  dispMessage(ptr);
  delay(dTime);
  ptr++;
  if (ptr==128) ptr=0;
  if (ptr==0 || ptr==64) getMessage(64-ptr);
}
